@@ -243,28 +243,6 @@ pdf_document_get_n_pages (EvDocument *document)
 	return poppler_document_get_n_pages (PDF_DOCUMENT (document)->document);
 }
 
-static void
-set_page_orientation (PdfDocument *pdf_document, PopplerPage *page, int rotation)
-{
-	PopplerOrientation orientation;
-
-	switch (rotation) {
-	case 90:
-		orientation = POPPLER_ORIENTATION_LANDSCAPE;
-		break;
-	case 180:
-		orientation = POPPLER_ORIENTATION_UPSIDEDOWN;
-		break;
-	case 270:
-		orientation = POPPLER_ORIENTATION_SEASCAPE;
-		break;
-	default:
-		orientation = POPPLER_ORIENTATION_PORTRAIT;
-	}
-
-	poppler_page_set_orientation (page, orientation);
-}
-
 static void
 pdf_document_get_page_size (EvDocument   *document,
 			    int           page,
@@ -349,11 +327,16 @@ pdf_document_render_pixbuf (EvDocument   *document,
 	pdf_document = PDF_DOCUMENT (document);
 
 	set_rc_data (pdf_document, rc);
-	set_page_orientation (pdf_document, POPPLER_PAGE (rc->data), rc->rotation);
 
 	poppler_page_get_size (POPPLER_PAGE (rc->data), &width_points, &height_points);
-	width = (int) ((width_points * rc->scale) + 0.5);
-	height = (int) ((height_points * rc->scale) + 0.5);
+
+	if (rc->rotation == 90 || rc->rotation == 270) {
+		width = (int) ((height_points * rc->scale) + 0.5);
+		height = (int) ((width_points * rc->scale) + 0.5);
+	} else {
+		width = (int) ((width_points * rc->scale) + 0.5);
+		height = (int) ((height_points * rc->scale) + 0.5);
+	}
 
 	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
 				 FALSE, 8,
@@ -363,6 +346,7 @@ pdf_document_render_pixbuf (EvDocument   *document,
 				       0, 0,
 				       width, height,
 				       rc->scale,
+				       rc->rotation,
 				       pixbuf);
 	
 	
@@ -718,19 +702,108 @@ pdf_document_links_has_document_links (EvDocumentLinks *document_links)
 	return TRUE;
 }
 
+static EvLink *
+ev_link_from_dest (PopplerAction *action)
+{
+	EvLink *link = NULL;
+	const char *unimplemented_dest = NULL;
+
+	switch (action->goto_dest.dest->type) {
+	case POPPLER_DEST_UNKNOWN:
+		unimplemented_dest = "POPPLER_DEST_UNKNOWN";
+		break;
+	case POPPLER_DEST_XYZ:
+		link = ev_link_new_page_xyz (action->any.title,
+					     action->goto_dest.dest->page_num - 1,
+					     action->goto_dest.dest->left,
+					     action->goto_dest.dest->top,
+					     action->goto_dest.dest->zoom);
+		break;
+	case POPPLER_DEST_FIT:
+		link = ev_link_new_page_fit (action->any.title,
+					     action->goto_dest.dest->page_num - 1);
+		break;
+	case POPPLER_DEST_FITH:
+		link = ev_link_new_page_fith (action->any.title,
+					      action->goto_dest.dest->page_num - 1,
+					      action->goto_dest.dest->top);
+		break;
+	case POPPLER_DEST_FITV:
+		link = ev_link_new_page_fitv (action->any.title,
+					      action->goto_dest.dest->page_num - 1,
+					      action->goto_dest.dest->left);
+		break;
+	case POPPLER_DEST_FITR:
+		link = ev_link_new_page_fitr (action->any.title,
+					      action->goto_dest.dest->page_num - 1,
+					      action->goto_dest.dest->left,
+					      action->goto_dest.dest->bottom,
+					      action->goto_dest.dest->right,
+					      action->goto_dest.dest->top);
+		break;
+	case POPPLER_DEST_FITB:
+		unimplemented_dest = "POPPLER_DEST_FITB";
+		break;
+	case POPPLER_DEST_FITBH:
+		unimplemented_dest = "POPPLER_DEST_FITBH";
+		break;
+	case POPPLER_DEST_FITBV:
+		unimplemented_dest = "POPPLER_DEST_FITBV";
+		break;
+	}
+
+	if (unimplemented_dest) {
+		g_warning ("Unimplemented destination: %s, please post a bug report with a testcase.",
+			   unimplemented_dest);
+	}
+
+	if (link == NULL) {
+		link = ev_link_new_page (action->any.title, action->goto_dest.dest->page_num - 1);
+	}
+
+	return link;
+}
+
 static EvLink *
 ev_link_from_action (PopplerAction *action)
 {
-	EvLink *link;
+	EvLink *link = NULL;
 	const char *title;
+	const char *unimplemented_action = NULL;
 
 	title = action->any.title;
-	
-	if (action->type == POPPLER_ACTION_GOTO_DEST) {
-		link = ev_link_new_page (title, action->goto_dest.dest->page_num - 1);
-	} else if (action->type == POPPLER_ACTION_URI) {
+
+	switch (action->type) {
+	case POPPLER_ACTION_UNKNOWN:
+		link = ev_link_new_title (title);
+		break;
+	case POPPLER_ACTION_GOTO_DEST:
+		link = ev_link_from_dest (action);
+		break;
+	case POPPLER_ACTION_GOTO_REMOTE:
+		unimplemented_action = "POPPLER_ACTION_GOTO_REMOTE";
+		break;
+	case POPPLER_ACTION_LAUNCH:
+		link = ev_link_new_launch (title, action->launch.file_name,
+					   action->launch.params);
+		break;
+	case POPPLER_ACTION_URI:
 		link = ev_link_new_external (title, action->uri.uri);
-	} else {
+		break;
+	case POPPLER_ACTION_NAMED:
+		unimplemented_action = "POPPLER_ACTION_NAMED";
+		break;
+	case POPPLER_ACTION_MOVIE:
+		unimplemented_action = "POPPLER_ACTION_MOVIE";
+		break;
+	}
+
+	if (unimplemented_action) {
+		g_warning ("Unimplemented action: %s, please post a bug report with a testcase.",
+			   unimplemented_action);
+	}
+
+	if (link == NULL) {
 		link = ev_link_new_title (title);
 	}
 
@@ -754,16 +827,22 @@ build_tree (PdfDocument      *pdf_document,
 		action = poppler_index_iter_get_action (iter);
 		expand = poppler_index_iter_is_open (iter);
 		if (action) {
+			char *title_markup;
+
 			gtk_tree_store_append (GTK_TREE_STORE (model), &tree_iter, parent);
 			link = ev_link_from_action (action);
 			poppler_action_free (action);
+			title_markup = g_markup_escape_text (ev_link_get_title (link), -1);
 
 			gtk_tree_store_set (GTK_TREE_STORE (model), &tree_iter,
-					    EV_DOCUMENT_LINKS_COLUMN_MARKUP, ev_link_get_title (link),
+					    EV_DOCUMENT_LINKS_COLUMN_MARKUP, title_markup,
 					    EV_DOCUMENT_LINKS_COLUMN_LINK, link,
 					    EV_DOCUMENT_LINKS_COLUMN_EXPAND, expand,
 					    -1);
+
+			g_free (title_markup);
 			g_object_unref (link);
+
 			child = poppler_index_iter_get_child (iter);
 			if (child)
 				build_tree (pdf_document, model, &tree_iter, child);
@@ -807,17 +886,15 @@ static GdkPixbuf *
 make_thumbnail_for_size (PdfDocument   *pdf_document,
 			 gint           page,
 			 int            rotation,
-			 gint           size,
-			 gboolean       border)
+			 gint           size)
 {
 	PopplerPage *poppler_page;
-	GdkPixbuf *pixbuf, *sub_pixbuf;
+	GdkPixbuf *pixbuf;
 	int width, height;
 	double scale;
 	gdouble unscaled_width, unscaled_height;
 
 	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	set_page_orientation (pdf_document, poppler_page, rotation);
 	g_return_val_if_fail (poppler_page != NULL, NULL);
 
 	pdf_document_thumbnails_get_dimensions (EV_DOCUMENT_THUMBNAILS (pdf_document), page,
@@ -825,28 +902,25 @@ make_thumbnail_for_size (PdfDocument   *pdf_document,
 	poppler_page_get_size (poppler_page, &unscaled_width, &unscaled_height);
 	scale = width / unscaled_width;
 
-	if (border) {
-		pixbuf = ev_document_misc_get_thumbnail_frame (width, height, NULL);
-
-		sub_pixbuf = gdk_pixbuf_new_subpixbuf (pixbuf,
-						       1, 1,
-						       width - 1, height - 1);
-	} else {
-		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-					 width, height);
-		gdk_pixbuf_fill (pixbuf, 0xffffffff);
-		sub_pixbuf = gdk_pixbuf_new_subpixbuf (pixbuf,
-						       0, 0,
-						       width, height);
+	/* rotate */
+	if (rotation == 90 || rotation == 270) {
+		int temp;
+		temp = width;
+		width = height;
+		height = temp;
 	}
 
+	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8,
+				 width, height);
+	gdk_pixbuf_fill (pixbuf, 0xffffffff);
+
 	poppler_page_render_to_pixbuf (poppler_page, 0, 0,
 				       width, height,
-				       scale, sub_pixbuf);
-
-	g_object_unref (G_OBJECT (sub_pixbuf));
+				       scale, rotation, pixbuf);
+       
 
 	g_object_unref (poppler_page);
+
 	return pixbuf;
 }
 
@@ -855,34 +929,31 @@ pdf_document_thumbnails_get_thumbnail (EvDocumentThumbnails *document_thumbnails
 	 			       gint 		     page,
 				       gint                  rotation,
  				       gint                  size,
-		 		       gboolean              border)
+				       gboolean              border)
 {
 	PdfDocument *pdf_document;
 	PopplerPage *poppler_page;
 	GdkPixbuf *pixbuf;
+	GdkPixbuf *border_pixbuf;
 
 	pdf_document = PDF_DOCUMENT (document_thumbnails);
 
 	poppler_page = poppler_document_get_page (pdf_document->document, page);
-	set_page_orientation (pdf_document, poppler_page, rotation);
 	g_return_val_if_fail (poppler_page != NULL, NULL);
 
 	pixbuf = poppler_page_get_thumbnail (poppler_page);
 	
-	if (pixbuf != NULL) {
-		/* The document provides its own thumbnails. */
-		if (border) {
-			GdkPixbuf *real_pixbuf;
-
-			real_pixbuf = ev_document_misc_get_thumbnail_frame (-1, -1, pixbuf);
-			g_object_unref (pixbuf);
-			pixbuf = real_pixbuf;
-		}
-	} else {
+	if (pixbuf == NULL) {
 		/* There is no provided thumbnail.  We need to make one. */
-		pixbuf = make_thumbnail_for_size (pdf_document, page, rotation, size, border);
+		pixbuf = make_thumbnail_for_size (pdf_document, page, rotation, size);
 	}
 
+        if (border) {		
+		border_pixbuf = ev_document_misc_get_thumbnail_frame (-1, -1, rotation, pixbuf);
+		g_object_unref (pixbuf);
+		pixbuf = border_pixbuf;
+	}		
+
 	g_object_unref (poppler_page);
 	
 	return pixbuf;
@@ -912,13 +983,8 @@ pdf_document_thumbnails_get_dimensions (EvDocumentThumbnails *document_thumbnail
 		double page_width, page_height;
 
 		poppler_page_get_size (poppler_page, &page_width, &page_height);
-		if (page_width > page_height) {
-			*width = size;
-			*height = (int) (size * page_height / page_width);
-		} else {
-			*width = (int) (size * page_width / page_height);
-			*height = size;
-		}
+		*width = size;
+		*height = (int) (size * page_height / page_width);
 	}
 	g_object_unref (poppler_page);
 }
@@ -1077,9 +1143,7 @@ pdf_document_find_page_has_results (EvDocumentFind *document_find,
 {
 	PdfDocumentSearch *search = PDF_DOCUMENT (document_find)->search;
 
-	g_return_val_if_fail (search != NULL, FALSE);
-
-	return search->pages[page] != NULL;
+	return search && search->pages[page] != NULL;
 }
 
 double
@@ -1151,7 +1215,6 @@ pdf_document_ps_exporter_do_page (EvPSExporter *exporter, EvRenderContext *rc)
 	g_return_if_fail (pdf_document->ps_file != NULL);
 
 	poppler_page = poppler_document_get_page (pdf_document->document, rc->page);
-	set_page_orientation (pdf_document, poppler_page, rc->rotation);
 	poppler_page_render_to_ps (poppler_page, pdf_document->ps_file);
 	g_object_unref (poppler_page);
 }
@@ -1190,8 +1253,6 @@ pdf_selection_render_selection (EvSelection      *selection,
 	pdf_document = PDF_DOCUMENT (selection);
 	set_rc_data (pdf_document, rc);
 
-	set_page_orientation (pdf_document, POPPLER_PAGE (rc->data), rc->rotation);
-
 	poppler_page_get_size (POPPLER_PAGE (rc->data), &width_points, &height_points);
 	width = (int) ((width_points * rc->scale) + 0.5);
 	height = (int) ((height_points * rc->scale) + 0.5);
@@ -1203,7 +1264,7 @@ pdf_selection_render_selection (EvSelection      *selection,
 	}
 	
 	poppler_page_render_selection (POPPLER_PAGE (rc->data),
-				       rc->scale, *pixbuf,
+				       rc->scale, rc->rotation, *pixbuf,
 				       (PopplerRectangle *)points,
 				       (PopplerRectangle *)old_points,
 				       text,
@@ -1222,7 +1283,6 @@ pdf_selection_get_selection_region (EvSelection     *selection,
 	pdf_document = PDF_DOCUMENT (selection);
 
 	set_rc_data (pdf_document, rc);
-	set_page_orientation (pdf_document, POPPLER_PAGE (rc->data), rc->rotation);
 
 	retval = poppler_page_get_selection_region ((PopplerPage *)rc->data, rc->scale, (PopplerRectangle *) points);
 
@@ -1241,7 +1301,6 @@ pdf_selection_get_selection_map (EvSelection     *selection,
 	pdf_document = PDF_DOCUMENT (selection);
 	poppler_page = poppler_document_get_page (pdf_document->document,
 						  rc->page);
-	set_page_orientation (pdf_document, poppler_page, rc->rotation);
 
 	points.x1 = 0.0;
 	points.y1 = 0.0;
