@@ -23,6 +23,8 @@
 #include <gtk/gtkwidget.h>
 #include "ev-document.h"
 
+#define DUAL_EVEN_LEFT 1
+
 G_BEGIN_DECLS
 #define EV_TYPE_PAGE_CACHE            (ev_page_cache_get_type ())
 #define EV_PAGE_CACHE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), EV_TYPE_PAGE_CACHE, EvPageCache))
@@ -66,10 +68,6 @@ void           ev_page_cache_set_current_page    (EvPageCache *page_cache,
 						  int          page);
 gboolean       ev_page_cache_set_page_label      (EvPageCache *page_cache,
 						  const char  *page_label);
-void           ev_page_cache_set_link            (EvPageCache *page_cache,
-						  EvLink      *link);
-gboolean       ev_page_cache_next_page           (EvPageCache *page_cache);
-gboolean       ev_page_cache_prev_page           (EvPageCache *page_cache);
 
 EvPageCache   *ev_page_cache_get		 (EvDocument *document);
 
