@@ -52,8 +52,13 @@ struct _EvSidebarLinksPrivate {
 enum {
 	PROP_0,
 	PROP_MODEL,
+	PROP_WIDGET,
 };
 
+enum {
+	LINK_ACTIVATED,
+	N_SIGNALS
+};
 
 static void links_page_num_func				(GtkTreeViewColumn *tree_column,
 							 GtkCellRenderer   *cell,
@@ -67,14 +72,16 @@ static void row_activated_callback 			(GtkTreeView *treeview,
 		                                         GtkTreePath *arg1,
 	                                                 GtkTreeViewColumn *arg2,
 		                                         gpointer user_data);
+static void job_finished_callback 			(EvJobLinks     *job,
+				    		         EvSidebarLinks *sidebar_links);
 static void ev_sidebar_links_page_iface_init 		(EvSidebarPageIface *iface);
-static void ev_sidebar_links_clear_document     	(EvSidebarLinks *sidebar_links);
 static void ev_sidebar_links_set_document      	 	(EvSidebarPage  *sidebar_page,
 		    			        	 EvDocument     *document);
 static gboolean ev_sidebar_links_support_document	(EvSidebarPage  *sidebar_page,
 						         EvDocument     *document);
 static const gchar* ev_sidebar_links_get_label 		(EvSidebarPage *sidebar_page);
 
+static guint signals[N_SIGNALS];
 
 G_DEFINE_TYPE_EXTENDED (EvSidebarLinks, 
                         ev_sidebar_links, 
@@ -87,15 +94,6 @@ G_DEFINE_TYPE_EXTENDED (EvSidebarLinks,
 #define EV_SIDEBAR_LINKS_GET_PRIVATE(object) \
 	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EV_TYPE_SIDEBAR_LINKS, EvSidebarLinksPrivate))
 
-
-static void
-ev_sidebar_links_destroy (GtkObject *object)
-{
-	EvSidebarLinks *ev_sidebar_links = (EvSidebarLinks *) object;
-
-	ev_sidebar_links_clear_document (ev_sidebar_links);
-}
-
 static void
 ev_sidebar_links_set_property (GObject      *object,
 			       guint         prop_id,
@@ -136,26 +134,61 @@ ev_sidebar_links_get_property (GObject    *object,
 	case PROP_MODEL:
 		g_value_set_object (value, ev_sidebar_links->priv->model);
 		break;
+	case PROP_WIDGET:
+		g_value_set_object (value, ev_sidebar_links->priv->tree_view);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
+static void
+ev_sidebar_links_dispose (GObject *object)
+{
+	EvSidebarLinks *sidebar = EV_SIDEBAR_LINKS (object);
+
+	if (sidebar->priv->job) {
+		g_signal_handlers_disconnect_by_func (sidebar->priv->job,
+						      job_finished_callback, sidebar);
+		ev_job_queue_remove_job (sidebar->priv->job);						      
+		g_object_unref (sidebar->priv->job);
+		sidebar->priv->job = NULL;
+	}
+
+	if (sidebar->priv->model) {
+		g_object_unref (sidebar->priv->model);
+		sidebar->priv->model = NULL;
+	}
+
+	if (sidebar->priv->document) {
+		g_object_unref (sidebar->priv->document);
+		sidebar->priv->document = NULL;
+		sidebar->priv->page_cache = NULL;
+	}
+
+
+	G_OBJECT_CLASS (ev_sidebar_links_parent_class)->dispose (object);
+}
 
 static void
 ev_sidebar_links_class_init (EvSidebarLinksClass *ev_sidebar_links_class)
 {
 	GObjectClass *g_object_class;
-	GtkObjectClass *gtk_object_class;
 
 	g_object_class = G_OBJECT_CLASS (ev_sidebar_links_class);
-	gtk_object_class = GTK_OBJECT_CLASS (ev_sidebar_links_class);
 
 	g_object_class->set_property = ev_sidebar_links_set_property;
 	g_object_class->get_property = ev_sidebar_links_get_property;
+	g_object_class->dispose = ev_sidebar_links_dispose;
 
-	gtk_object_class->destroy = ev_sidebar_links_destroy;
+	signals[LINK_ACTIVATED] = g_signal_new ("link-activated",
+			 G_TYPE_FROM_CLASS (g_object_class),
+		         G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		         G_STRUCT_OFFSET (EvSidebarLinksClass, link_activated),
+		         NULL, NULL,
+		         g_cclosure_marshal_VOID__OBJECT,
+		         G_TYPE_NONE, 1, G_TYPE_OBJECT);
 
 	g_object_class_install_property (g_object_class,
 					 PROP_MODEL,
@@ -164,6 +197,9 @@ ev_sidebar_links_class_init (EvSidebarLinksClass *ev_sidebar_links_class)
 							      "Current Model",
 							      GTK_TYPE_TREE_MODEL,
 							      G_PARAM_READWRITE));
+	g_object_class_override_property (g_object_class,
+					  PROP_WIDGET,
+					  "main-widget");
 
 	g_type_class_add_private (g_object_class, sizeof (EvSidebarLinksPrivate));
 }
@@ -193,10 +229,11 @@ selection_changed_callback (GtkTreeSelection   *selection,
 
 		g_signal_handler_block (ev_sidebar_links->priv->page_cache,
 					ev_sidebar_links->priv->page_changed_id);
-		/* FIXME: we should handle this better.  This breaks w/ URLs */
-		ev_page_cache_set_link (ev_sidebar_links->priv->page_cache, link);
+		g_signal_emit (ev_sidebar_links, signals[LINK_ACTIVATED], 0, link);
 		g_signal_handler_unblock (ev_sidebar_links->priv->page_cache,
 					  ev_sidebar_links->priv->page_changed_id);
+
+		g_object_unref (link);
 	}
 }
 
@@ -210,12 +247,14 @@ create_loading_model (void)
 	/* Creates a fake model to indicate that we're loading */
 	retval = (GtkTreeModel *)gtk_list_store_new (EV_DOCUMENT_LINKS_COLUMN_NUM_COLUMNS,
 						     G_TYPE_STRING,
-						     G_TYPE_OBJECT);
+						     G_TYPE_OBJECT,
+						     G_TYPE_BOOLEAN);
 
 	gtk_list_store_append (GTK_LIST_STORE (retval), &iter);
 	markup = g_strdup_printf ("<span size=\"larger\" style=\"italic\">%s</span>", _("Loading..."));
 	gtk_list_store_set (GTK_LIST_STORE (retval), &iter,
 			    EV_DOCUMENT_LINKS_COLUMN_MARKUP, markup,
+			    EV_DOCUMENT_LINKS_COLUMN_EXPAND, FALSE,
 			    EV_DOCUMENT_LINKS_COLUMN_LINK, NULL,
 			    -1);
 	g_free (markup);
@@ -243,13 +282,19 @@ print_section_cb (GtkWidget *menuitem, EvSidebarLinks *sidebar)
 				    -1);
 		first_page = ev_link_get_page (link) + 1;
 
+		if (link)
+			g_object_unref (link);
+
 		if (gtk_tree_model_iter_next (model, &iter)) {
 			gtk_tree_model_get (model, &iter,
 					    EV_DOCUMENT_LINKS_COLUMN_LINK, &link,
 					    -1);
 			last_page = ev_link_get_page (link);
+
+			if (link)
+		    		g_object_unref (link);
 		} else {
-			last_page = -1;
+			last_page =  ev_page_cache_get_n_pages (sidebar->priv->page_cache);
 		}
 	
 		window = gtk_widget_get_toplevel (GTK_WIDGET (sidebar));
@@ -401,7 +446,7 @@ links_page_num_func (GtkTreeViewColumn *tree_column,
 			    -1);
 	
 	if (link != NULL &&
-	    ev_link_get_link_type (link) == EV_LINK_TYPE_PAGE) {
+	    ev_link_get_page (link) >= 0) {
 		gchar *page_label;
 		gchar *page_string;
 
@@ -420,6 +465,9 @@ links_page_num_func (GtkTreeViewColumn *tree_column,
 			      "visible", FALSE,
 			      NULL);
 	}
+
+	if (link)
+		g_object_unref (link);
 }
 
 /* Public Functions */
@@ -434,24 +482,6 @@ ev_sidebar_links_new (void)
 	return ev_sidebar_links;
 }
 
-static void
-ev_sidebar_links_clear_document (EvSidebarLinks *sidebar_links)
-{
-	EvSidebarLinksPrivate *priv;
-
-	g_return_if_fail (EV_IS_SIDEBAR_LINKS (sidebar_links));
-
-	priv = sidebar_links->priv;
-
-	if (priv->document) {
-		g_object_unref (priv->document);
-		priv->document = NULL;
-		priv->page_cache = NULL;
-	}
-
-	gtk_tree_view_set_model (GTK_TREE_VIEW (priv->tree_view), NULL);
-}
-
 static gboolean
 update_page_callback_foreach (GtkTreeModel *model,
 			      GtkTreePath  *path,
@@ -465,7 +495,7 @@ update_page_callback_foreach (GtkTreeModel *model,
 			    EV_DOCUMENT_LINKS_COLUMN_LINK, &link,
 			    -1);
 
-	if (link && ev_link_get_link_type (link) == EV_LINK_TYPE_PAGE) {
+	if (link) {
 		int current_page;
 
 		current_page = ev_page_cache_get_current_page (sidebar_links->priv->page_cache);
@@ -473,13 +503,14 @@ update_page_callback_foreach (GtkTreeModel *model,
 			GtkTreeSelection *selection;
 
 			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sidebar_links->priv->tree_view));
-
 			gtk_tree_selection_select_path (selection, path);
-
+			g_object_unref (link);
+			
 			return TRUE;
 		}
+		g_object_unref (link);
 	}
-	
+
 	return FALSE;
 }
 
@@ -489,19 +520,37 @@ update_page_callback (EvPageCache    *page_cache,
 		      EvSidebarLinks *sidebar_links)
 {
 	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sidebar_links->priv->tree_view));
+
+	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		EvLink *link;
+
+		gtk_tree_model_get (model, &iter,
+				    EV_DOCUMENT_LINKS_COLUMN_LINK, &link,
+				    -1);
+		if (link) {
+			gint current_page;		
+			current_page = ev_page_cache_get_current_page (sidebar_links->priv->page_cache);
+			if (ev_link_get_page (link) == current_page) {
+				g_object_unref (link);
+				return;
+			}
+			g_object_unref (link);
+		}
+	}		
+
 	/* We go through the tree linearly looking for the first page that
 	 * matches.  This is pretty inefficient.  We can do something neat with
 	 * a GtkTreeModelSort here to make it faster, if it turns out to be
 	 * slow.
 	 */
-
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (sidebar_links->priv->tree_view));
-
 	g_signal_handler_block (selection, sidebar_links->priv->selection_id);
 	g_signal_handler_block (sidebar_links->priv->tree_view, sidebar_links->priv->row_activated_id);
 
-	gtk_tree_selection_unselect_all (selection);
-	gtk_tree_model_foreach (sidebar_links->priv->model,
+	gtk_tree_model_foreach (model,
 				update_page_callback_foreach,
 				sidebar_links);
 
@@ -523,35 +572,50 @@ row_activated_callback 			(GtkTreeView *treeview,
 	
         return;
 }
-				
+
+static void
+expand_open_links (GtkTreeView *tree_view, GtkTreeModel *model, GtkTreeIter *parent)
+{
+	GtkTreeIter iter;
+	gboolean expand;
+
+	if (gtk_tree_model_iter_children (model, &iter, parent)) {
+		do {
+			gtk_tree_model_get (model, &iter,
+					    EV_DOCUMENT_LINKS_COLUMN_EXPAND, &expand,
+					    -1);
+			if (expand) {
+				GtkTreePath *path;
+
+				path = gtk_tree_model_get_path (model, &iter);
+				gtk_tree_view_expand_row (tree_view, path, FALSE);
+				gtk_tree_path_free (path);
+			}
+
+			expand_open_links (tree_view, model, &iter);
+		} while (gtk_tree_model_iter_next (model, &iter));
+	}
+}
+	
 static void
 job_finished_callback (EvJobLinks     *job,
 		       EvSidebarLinks *sidebar_links)
 {
 	EvSidebarLinksPrivate *priv;
 	GtkTreeSelection *selection;
-	GtkTreeIter iter;
-	GtkTreePath *path;
-	gboolean result;
 
 	priv = sidebar_links->priv;
-
-	priv->model = g_object_ref (job->model);
+	
+	priv->model = job->model;
 	g_object_notify (G_OBJECT (sidebar_links), "model");
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (priv->tree_view), job->model);
-	g_object_unref (job);
-
-	/* Expand one level of the tree */
-	path = gtk_tree_path_new_first ();
-	for (result = gtk_tree_model_get_iter_first (priv->model, &iter);
-	     result;
-	     result = gtk_tree_model_iter_next (priv->model, &iter)) {
-		gtk_tree_view_expand_row (GTK_TREE_VIEW (priv->tree_view), path, FALSE);
-		gtk_tree_path_next (path);
-	}
-	gtk_tree_path_free (path);
 	
+	g_object_unref (job);
+	priv->job = NULL;
+
+	expand_open_links (GTK_TREE_VIEW (priv->tree_view), priv->model, NULL);
+
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree_view));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 	priv->selection_id = g_signal_connect (selection, "changed",
@@ -582,10 +646,20 @@ ev_sidebar_links_set_document (EvSidebarPage  *sidebar_page,
 
 	priv = sidebar_links->priv;
 
-	g_object_ref (document);
+	if (priv->document) {
+		gtk_tree_view_set_model (GTK_TREE_VIEW (priv->tree_view), NULL);
+		g_object_unref (priv->document);
+	}
 
-	priv->document = document;
-	priv->page_cache = ev_document_get_page_cache (document);
+	priv->document = g_object_ref (document);
+	priv->page_cache = ev_page_cache_get (document);
+
+	if (priv->job) {
+		g_signal_handlers_disconnect_by_func (priv->job,
+						      job_finished_callback,
+						      sidebar_links);
+		g_object_unref (priv->job);
+	}
 
 	priv->job = ev_job_links_new (document);
 	g_signal_connect (priv->job,
@@ -594,7 +668,6 @@ ev_sidebar_links_set_document (EvSidebarPage  *sidebar_page,
 			  sidebar_links);
 	/* The priority doesn't matter for this job */
 	ev_job_queue_add_job (priv->job, EV_JOB_PRIORITY_LOW);
-
 }
 
 static gboolean
@@ -611,12 +684,6 @@ ev_sidebar_links_get_label (EvSidebarPage *sidebar_page)
     return _("Index");
 }
 
-GtkWidget *
-ev_sidebar_links_get_treeview (EvSidebarLinks *sidebar)
-{
-	return sidebar->priv->tree_view;
-}
-
 static void
 ev_sidebar_links_page_iface_init (EvSidebarPageIface *iface)
 {
