@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: main.c,v 1.22 2005/07/11 11:20:42 carlosgc Exp $
+ *  $Id: main.c,v 1.28 2005/11/10 01:02:54 nshmyrev Exp $
  */
 
 #include "config.h"
@@ -24,7 +24,10 @@
 #include "ev-metadata-manager.h"
 
 #include <glib/gi18n.h>
+#include <gdk/gdkx.h>
 #include <gtk/gtkmain.h>
+#include <stdlib.h>
+#include <string.h>
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-app-helper.h>
@@ -54,7 +57,7 @@ load_files (const char **files)
 	int i;
 
 	if (!files) {
-		ev_application_open_window (EV_APP, NULL);
+		ev_application_open_window (EV_APP, GDK_CURRENT_TIME, NULL);
 		return;
 	}
 
@@ -62,12 +65,55 @@ load_files (const char **files)
 		char *uri;
 
 		uri = gnome_vfs_make_uri_from_shell_arg (files[i]);
-		ev_application_open_uri (EV_APP, uri, ev_page_label, NULL);		
+		ev_application_open_uri (EV_APP, uri, ev_page_label,
+					 GDK_CURRENT_TIME, NULL);
 		g_free (uri);
         }
 }
 
 #ifdef ENABLE_DBUS
+
+#ifndef HAVE_GTK_WINDOW_PRESENT_WITH_TIME
+static guint32
+get_startup_time (void)
+{
+	const char *envvar, *timestamp;
+	unsigned long value;
+	char *end;
+
+	envvar = getenv ("DESKTOP_STARTUP_ID");
+
+	if (envvar == NULL)
+		return 0;
+
+/* DESKTOP_STARTUP_ID is of form "<unique>_TIME<timestamp>".
+ *
+ * <unique> might contain a T but <timestamp> is an integer.  As such,
+ * the last 'T' in the string must be the start of "TIME".
+ */
+	timestamp = rindex (envvar, 'T');
+
+/* Maybe the word "TIME" was not found... */
+	if (timestamp == NULL || strncmp (timestamp, "TIME", 4))
+		return 0;
+
+	timestamp += 4;
+
+/* strtoul sets errno = ERANGE on overflow, but it is not specified
+ * if it sets it to 0 on success.  Doing so ourselves is the only
+ * way to know for sure.
+ */
+	errno = 0;
+	value = strtoul (timestamp, &end, 10);
+
+/* unsigned long might be 64bit, so double-check! */
+	if (errno != 0 || *end != '\0' || value > G_MAXINT32)
+		return 0;
+
+	return value;
+}
+#endif
+
 static gboolean
 load_files_remote (const char **files)
 {
@@ -79,8 +125,19 @@ load_files_remote (const char **files)
 	DBusGPendingCall *call;
 #endif
 	DBusGProxy *remote_object;
+#ifdef HAVE_GTK_WINDOW_PRESENT_WITH_TIME
+	GdkDisplay *display;
+#endif
+	guint32 timestamp;
 
-	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+#ifdef HAVE_GTK_WINDOW_PRESENT_WITH_TIME
+	display = gdk_display_get_default();
+	timestamp = gdk_x11_display_get_user_time (display);
+#else
+	/* Fake it for GTK+2.6 */
+	timestamp = get_startup_time ();
+#endif
+	connection = dbus_g_bus_get (DBUS_BUS_STARTER, &error);
 	if (connection == NULL) {
 		g_warning (error->message);
 		g_error_free (error);	
@@ -94,7 +151,9 @@ load_files_remote (const char **files)
                                                    "org.gnome.evince.Application");
 	if (!files) {
 #if DBUS_VERSION <= 33
-		call = dbus_g_proxy_begin_call (remote_object, "OpenWindow", DBUS_TYPE_INVALID);
+		call = dbus_g_proxy_begin_call (remote_object, "OpenWindow",
+						DBUS_TYPE_UINT32, &timestamp,
+						DBUS_TYPE_INVALID);
 
 		if (!dbus_g_proxy_end_call (remote_object, call, &error, DBUS_TYPE_INVALID)) {
 			g_warning (error->message);
@@ -102,7 +161,9 @@ load_files_remote (const char **files)
 			return FALSE;
 		}
 #elif DBUS_VERSION == 34
-		call = dbus_g_proxy_begin_call (remote_object, "OpenWindow", G_TYPE_INVALID);
+		call = dbus_g_proxy_begin_call (remote_object, "OpenWindow",
+						G_TYPE_UINT, timestamp,
+						G_TYPE_INVALID);
 
 		if (!dbus_g_proxy_end_call (remote_object, call, &error, G_TYPE_INVALID)) {
 			g_warning (error->message);
@@ -110,7 +171,10 @@ load_files_remote (const char **files)
 			return FALSE;
 		}
 #else
-		if (!dbus_g_proxy_call (remote_object, "OpenWindow", &error, G_TYPE_INVALID)) {
+		if (!dbus_g_proxy_call (remote_object, "OpenWindow", &error,
+					G_TYPE_UINT, timestamp,
+					G_TYPE_INVALID,
+					G_TYPE_INVALID)) {
 			g_warning (error->message);
 			g_clear_error (&error);
 			return FALSE;
@@ -129,6 +193,7 @@ load_files_remote (const char **files)
 		call = dbus_g_proxy_begin_call (remote_object, "OpenURI",
 						DBUS_TYPE_STRING, &uri,
 						DBUS_TYPE_STRING, &page_label,
+						DBUS_TYPE_UINT32, &timestamp,
 						DBUS_TYPE_INVALID);
 
 		if (!dbus_g_proxy_end_call (remote_object, call, &error, DBUS_TYPE_INVALID)) {
@@ -141,6 +206,7 @@ load_files_remote (const char **files)
 		call = dbus_g_proxy_begin_call (remote_object, "OpenURI",
 						G_TYPE_STRING, uri,
 						G_TYPE_STRING, page_label,
+						G_TYPE_UINT, timestamp,
 						G_TYPE_INVALID);
 
 		if (!dbus_g_proxy_end_call (remote_object, call, &error, G_TYPE_INVALID)) {
@@ -153,6 +219,8 @@ load_files_remote (const char **files)
 		if (!dbus_g_proxy_call (remote_object, "OpenURI", &error,
 					G_TYPE_STRING, uri,
 					G_TYPE_STRING, page_label,
+					G_TYPE_UINT, timestamp,
+					G_TYPE_INVALID,
 					G_TYPE_INVALID)) {
 			g_warning (error->message);
 			g_clear_error (&error);
@@ -164,6 +232,8 @@ load_files_remote (const char **files)
 		result = TRUE;
         }
 
+	gdk_notify_startup_complete ();
+
 	return result;
 }
 #endif /* ENABLE_DBUS */
@@ -198,11 +268,9 @@ main (int argc, char *argv[])
 #ifdef ENABLE_DBUS
 	if (!ev_application_register_service (EV_APP)) {
 		if (load_files_remote (poptGetArgs (context))) {
-			g_warning ("Another process was running.");
 			return 0;
 		}
 	} else {
-		g_warning ("Starting evince process.");
 		enable_metadata = TRUE;
 	}
 #endif
